/**
 * 
 */
package luj.tools.BookBuyer;

import android.app.AlertDialog;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.CheckBox;
import android.widget.ImageButton;
import android.widget.TextView;

/**
 * @author luj
 * 
 */
public class BookTableAdapter extends BaseAdapter
{
	// view holder
	public final class ViewHolder
	{
		public CheckBox		select;
		public TextView		isbn;
		public TextView		name;
		public TextView		price;
		public TextView[]	shops	= new TextView[4];
		public ImageButton	delete;
	}

	// a tag to CameraPreview log message
	private static final String	TAG			= "BookTableAdapter";

	// initial size of row
	private final int			ROW_SIZE	= 4;

	// layout inflater
	private LayoutInflater		mInfolater;
	
	// Handler for updating price statistics
	private Handler 			mUpdateUIHandler;

	// constructor
	public BookTableAdapter(Context context, Handler handler)
	{
		mInfolater = LayoutInflater.from(context);
		mUpdateUIHandler = handler;
	}

	public int getCount()
	{
		int iBookCount = Library.getInstance().getBookCount();
		return iBookCount < ROW_SIZE ? ROW_SIZE : iBookCount;
	}

	public Object getItem(int arg0)
	{
		return null;
	}

	public long getItemId(int position)
	{
		return 0;
	}

	public View getView(final int position, View convertView, ViewGroup parent)
	{
		// set empty raws
		if (position >= Library.getInstance().getBookCount())
		{
			if (convertView == null || convertView.getTag() != null)
			{
				convertView = mInfolater.inflate(R.layout.book_table_empty_row,
						null);
			}
			return convertView;
		}

		// set raws that have content
		ViewHolder holder;
		if (convertView == null || convertView.getTag() == null)
		{
			convertView = mInfolater.inflate(R.layout.book_table_normal_row, null);
			holder = new ViewHolder();
			holder.select = (CheckBox) convertView
					.findViewById(R.id.Book_Table_Cell_Selected);
			holder.isbn = (TextView) convertView
					.findViewById(R.id.Book_Table_Cell_ISBN);
			holder.name = (TextView) convertView
					.findViewById(R.id.Book_Table_Cell_Name);
			holder.price = (TextView) convertView
					.findViewById(R.id.Book_Table_Cell_Martket_Price);
			holder.shops[0] = (TextView) convertView
					.findViewById(R.id.Book_Table_Cell_Shop0_Price);
			holder.shops[1] = (TextView) convertView
					.findViewById(R.id.Book_Table_Cell_Shop1_Price);
			holder.shops[2] = (TextView) convertView
					.findViewById(R.id.Book_Table_Cell_Shop2_Price);
			holder.shops[3] = (TextView) convertView
					.findViewById(R.id.Book_Table_Cell_Shop3_Price);
			holder.delete = (ImageButton) convertView
					.findViewById(R.id.Book_Table_Cell_Delete_Button);
			convertView.setTag(holder);
		}
		else
		{
			holder = (ViewHolder) convertView.getTag();
		}

		// show every child
		final Book book = Library.getInstance().getBook(position);
		if (book != null)
		{
			Log.v(TAG, "convertView book = " + position);
		}
		else
		{
			Log.v(TAG, "convertView book = null");
			return convertView;
		}

		// CheckBox for select
		holder.select.setChecked(book.getSelected());
		holder.select.setOnClickListener(new CheckBox.OnClickListener()
		{
			public void onClick(View v)
			{
				Book book = Library.getInstance().getBook(position);
				if (book != null)
				{
					Log.v(TAG, "select book = " + position);
					book.setSelected(!book.getSelected());
					mUpdateUIHandler.sendMessage(new Message());
				}
				else
				{
					Log.v(TAG, "select book == null!!");
					return;
				}
			}
		});

		// ISBN
		holder.isbn.setText(book.getBasicInfor().getISBN());
		holder.isbn.setOnClickListener(new TextView.OnClickListener()
		{
			public void onClick(View v)
			{
				Log.v("TextView", "ISBN click @" + position);
				Book book = Library.getInstance().getBook(position);
				if (book == null)
				{
					Log.v(TAG, "isbn diallog book == null!!");
					return;
				}
				else
				{
					Log.v(TAG, "isbn diallog book = " + position);
				}
				
				LayoutInflater inflater = LayoutInflater.from(v.getContext());
				View view = inflater.inflate(R.layout.dialog_book_information, null);
				TextView txtISBN = (TextView) view.findViewById(R.id.BookInforDialog_ISBN_Value);
				txtISBN.setText(book.getBasicInfor().getISBN());
				TextView txtBookname = (TextView) view.findViewById(R.id.BookInforDialog_Bookname_Value);
				txtBookname.setText(book.getBasicInfor().getName());
				TextView txtAuthor = (TextView) view.findViewById(R.id.BookInforDialog_Author_Value);
				txtAuthor.setText(book.getBasicInfor().getAuthor());
				TextView txtPublisher = (TextView) view.findViewById(R.id.BookInforDialog_Publisher_Value);
				txtPublisher.setText(book.getBasicInfor().getPublisher());
				TextView txtPrice = (TextView) view.findViewById(R.id.BookInforDialog_Price_Value);
				float fPrice = book.getBasicInfor().getPrice();
				String strPrice = "";
				if (fPrice > 0)
				{
					strPrice = String.format("%s%1.2f", book.getBasicInfor().getPriceUnit(), fPrice);
				}
				txtPrice.setText(strPrice);
				AlertDialog.Builder builder = new AlertDialog.Builder(v.getContext());
				builder.setTitle(v.getContext().getResources().getString(R.string.BookInforDialog_Tile));  
				builder.setView(view);  
				builder.setPositiveButton(v.getContext().getResources().getString(R.string.BookInforDialog_Button_OK), null);
				builder.show();
			}
		});

		// name
		holder.name.setText(book.getBasicInfor().getName());

		// price
		if (book.getBasicInfor().getPrice() > 0)
		{
			holder.price.setText(String.format("%1.2f", book.getBasicInfor()
					.getPrice()));
		}
		else
		{
			holder.price.setText("");
		}

		// discount price for each shop;
		for (int i = 0; i < GlobalOption.MAX_NUMBER_SHOP; ++i)
		{
			if (i >= GlobalOption.getInstance().getSelectedShops().size())
			{
				holder.shops[i].setText("");
				holder.shops[i].setOnLongClickListener(null);
				continue;
			}
			
			final String shopName = GlobalOption.getInstance().getSelectedShops().get(i);
			float price = book.getSaleInfor(shopName).getPrice();
			if (price > 0)
			{
				holder.shops[i].setText(String.format("%1.2f", price));
			}
			else
			{
				holder.shops[i].setText("");
			}
			
			holder.shops[i].setOnLongClickListener(new View.OnLongClickListener()
			{
				
				public boolean onLongClick(View v)
				{
					String link = book.getSaleInfor(shopName).getLink();
					if (link!=null && link.length()>0)
					{
						Log.v(TAG, "link = " + link);
						try
						{
							Uri uri = Uri.parse(link);
							Intent intent = new Intent(Intent.ACTION_VIEW, uri);
							v.getContext().startActivity(intent);
						}
						catch (Exception e)
						{
							e.printStackTrace();
						}
					}			
					return true;
				}
			});
		}

		// ImgaeButton for delete
		holder.delete.setOnClickListener(new ImageButton.OnClickListener()
		{
			public void onClick(View v)
			{
				Log.v("ImageButton", "click on " + position);
				Library.getInstance().deleteBook(position);
				mUpdateUIHandler.sendMessage(new Message());
			}
		});

		// colorful for odd/even row
		int color;
		if (position % 2 == 0)
		{
			color = convertView.getContext().getResources().getColor(
					R.color.BookTable_Background_EvenRow);
		}
		else
		{
			color = convertView.getContext().getResources().getColor(
					R.color.BookTable_Background_OddRow);
		}
		convertView.setBackgroundColor(color);

		return convertView;
	}
}