package luj.tools.BookBuyer;

import luj.tools.BookBuyer.Book.BookBasicInfor;
import luj.tools.BookBuyer.Book.BookSaleInfor;
import luj.tools.BookBuyer.ShopConfiguration.InformationElement;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;

import javax.xml.transform.Result;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;

import org.apache.http.NameValuePair;
import org.apache.http.message.BasicNameValuePair;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.tidy.Tidy;

import android.util.Log;

public class ShopInspector
{
	// a tag to CameraPreview log message
	private static final String	TAG	= "ShopInspector";

	public ShopInspector(ShopConfiguration config)
	{
		mConfig = config;
	}

	public boolean Inquire(BookBasicInfor bookInfor, BookSaleInfor saleInfor)
	{
		mBookInfor = bookInfor;
		mSaleInfor = saleInfor;

		String html = GetHTMLPage();
		if (html == null || html.length() == 0)
		{
			return false;
		}
		else
		{
			if (mConfig.needJumpPage())
			{
				html = JumpPage(html);
				if (html == null || html.length() == 0)
				{
					return false;
				}
				Log.v(TAG, "JumpPage() OK!");
			}
			return ParseHTMLPage(html);
		}
	}

	protected String JumpPage(String html)
	{
		// convert to XML structure and to get the root node
		Element rootElement = TransferHtmlToXml(html);
		if (rootElement == null)
		{
			Log.v(TAG, "TransferHtmlToXml() in JumpPage() fails");
			return null;
		}

		// get the jump node
		InformationElement jumpInfoElement = mConfig.getJumpElement();
		Element JumpXmlElement = (Element) XMLParseHelper.SelectSingleNode(
				jumpInfoElement.getPath(), rootElement);
		if (JumpXmlElement == null)
		{
			Log.v(TAG, "get Jump element fails!");
			return null;
		}

		// get the jump URL, and then get jump page
		if (jumpInfoElement.getAttribute() != null)
		{
			String link = JumpXmlElement.getAttribute(jumpInfoElement
					.getAttribute());
			mSaleInfor.setLink(link);
			return HttpConnectHelper.executeGetCommand(link);
		}

		return null;
	}

	protected String GetHTMLPage()
	{
		if (mConfig.isHttpGetProtocol())
		{
			String link = mConfig.getHttpGetLink().replace("token_isbn",
					mBookInfor.getISBN());
			return HttpConnectHelper.executeGetCommand(link);
		}
		else if (mConfig.isHttpPostProtocol())
		{
			List<NameValuePair> params = new ArrayList<NameValuePair>();
			for (int i = 0; i < mConfig.getHttpPostParames().size(); ++i)
			{
				String key = mConfig.getHttpPostParames().get(i).getName();
				String value = mConfig.getHttpPostParames().get(i).getValue();

				if (value.equalsIgnoreCase("token_isbn"))
				{
					params
							.add(new BasicNameValuePair(key, mBookInfor
									.getISBN()));
				}
				else
				{
					params.add(new BasicNameValuePair(key, value));
				}
			}

			return HttpConnectHelper.executePostCommand(mConfig
					.getHttpPostLink(), params);
		}

		return null;
	}

	private boolean ParseHTMLPage(String html)
	{
		// convert to XML structure and to get the root node
		Element rootElement = TransferHtmlToXml(html);
		if (rootElement == null)
		{
			Log.v(TAG, "TransferHtmlToXml() in ParseHTMLPage() fails");
			return false;
		}

		// to get book node
		String bookPath = mConfig.getBookElement().getPath();
		if (bookPath == null || bookPath.length() == 0)
		{
			return false;
		}
		Element elementBook = (Element) XMLParseHelper.SelectSingleNode(
				bookPath, rootElement);
		if (elementBook == null)
		{
			Log.v(TAG, "get Book element fails!");
			return false;
		}
		if (!ParseBookElement(elementBook))
		{
			return false;
		}

		return true;
	}

	// convert to XML structure and return the root node
	private Element TransferHtmlToXml(String html)
	{
		Document doc = null;
		Element rootElement = null;
		try
		{
			Tidy t = new Tidy();
			t.setInputEncoding("UTF-8");
			InputStream sin;
			sin = new ByteArrayInputStream(html.getBytes("UTF-8"));
			doc = t.parseDOM(sin, null);
			doc.normalize();
			rootElement = doc.getDocumentElement();
			//DebugOutString(html, "/sdcard/result.html");
			//DebugOutXmlDocument(doc, "/sdcard/result.xml");
			return rootElement;
		}
		catch (UnsupportedEncodingException e1)
		{
			Log.v(TAG, "Tidy.parseDOM() fails!!");
			e1.printStackTrace();
			return null;
		}
	}

	private void DebugOutString(String content, String fileName)
	{
		try
		{
			FileOutputStream fOutput = new FileOutputStream(fileName);
			fOutput.write(content.getBytes());
			fOutput.close();
			Log.v(TAG, "DebugOutString() Ok, file_name="+fileName);
			return;
		}
		catch (FileNotFoundException e)
		{
			e.printStackTrace();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
		Log.v(TAG, "DebugOutString() fails, file_name="+fileName);
	}

	private void DebugOutXmlDocument(Document doc, String fileName)
	{
		try
		{
			// Prepare the DOM document for writing
			Source source = new DOMSource(doc);

			// Prepare the output file
			Result result = new StreamResult(new File(fileName));

			// Write the DOM document to the file
			Transformer xformer = TransformerFactory.newInstance()
					.newTransformer();
			xformer.transform(source, result);
			Log.v(TAG, "DebugOutXmlDocument() Ok, file_name="+fileName);
			return;
		}
		catch (TransformerConfigurationException e)
		{
			e.printStackTrace();
		}
		catch (TransformerException e)
		{
			e.printStackTrace();
		}
		Log.v(TAG, "DebugOutXmlDocument() fails, file_name="+fileName);
	}

	private boolean ParseBookElement(Element elementBook)
	{
		CheckISBN(elementBook);

		ParseBooknameElement(elementBook);

		ParseAuthorElement(elementBook);

		ParsePublisherElement(elementBook);

		if (!mConfig.needJumpPage())
		{
			ParseLinkElement(elementBook);
		}

		ParsePriceElement(elementBook);

		ParseDiscountPriceElement(elementBook);

		return true;
	}

	private boolean CheckISBN(Element elementParent)
	{
		String strValue = ParseStringFromElement(elementParent, mConfig
				.getISBNElement());
		if (strValue != null)
		{
			Log.v(mConfig.getName() + ": ", "ISBN_String = " + strValue);
			if (strValue.indexOf(mBookInfor.getISBN()) != -1)
			{
				return true;
			}
		}

		Log.v(mConfig.getName() + ": ", "can't not found ISBN_String");
		return false;
	}

	private boolean ParseBooknameElement(Element elementParent)
	{
		if (mBookInfor.getName() != null && mBookInfor.getName().length() > 0
				&& mBookInfor.getPriority() >= mConfig.getPriority())
		{
			Log.v("book name : ", "the existing is higher priority");
			return true;
		}
		
		String strValue = ParseStringFromElement(elementParent, mConfig
				.getBooknamElement());
		if (strValue != null)
		{
			mBookInfor.setName(strValue);
			mBookInfor.setPriority(mConfig.getPriority());
			Log.v(mConfig.getName() + ": ", "Book_Name = " + strValue);
			return true;
		}

		Log.v(mConfig.getName() + ": ", "can't not found Book_Name");
		return false;
	}

	private boolean ParseAuthorElement(Element elementParent)
	{
		if (mBookInfor.getAuthor() != null && mBookInfor.getAuthor().length() > 0
				&& mBookInfor.getPriority() >= mConfig.getPriority())
		{
			Log.v("author : ", "the existing is higher priority");
			return true;
		}
		
		String strValue = ParseStringFromElement(elementParent, mConfig
				.getAuthorlement());
		if (strValue != null)
		{
			mBookInfor.setAuthor(strValue);
			mBookInfor.setPriority(mConfig.getPriority());
			Log.v(mConfig.getName() + ": ", "Author = " + strValue);
			return true;
		}

		Log.v(mConfig.getName() + ": ", "can't not found Author");
		return false;
	}

	private boolean ParsePublisherElement(Element elementParent)
	{		
		if (mBookInfor.getPublisher() != null && mBookInfor.getPublisher().length() > 0
				&& mBookInfor.getPriority() >= mConfig.getPriority())
		{
			Log.v("publisher : ", "the existing is higher priority");
			return true;
		}
		
		String strValue = ParseStringFromElement(elementParent, mConfig
				.getPublisherElement());
		if (strValue != null)
		{	
			mBookInfor.setPublisher(strValue);
			mBookInfor.setPriority(mConfig.getPriority());
			Log.v(mConfig.getName() + ": ", "Publisher = " + strValue);
			return true;
		}

		Log.v(mConfig.getName() + ": ", "can't not found Publisher");
		return false;
	}

	private boolean ParseLinkElement(Element elementParent)
	{
		String strValue = ParseStringFromElement(elementParent, mConfig
				.getLinkElement());
		if (strValue != null)
		{
			Log.v(mConfig.getName() + ": ", "Link1 = " + strValue);
			if (strValue.toLowerCase().indexOf("http") < 0)
			{
				String source;
				if (mConfig.isHttpGetProtocol())
				{
					source = mConfig.getHttpGetLink();
				}
				else
				{
					source = mConfig.getHttpPostLink();
				}
				int pos = source.indexOf('?');
				String prefix = source.substring(0, pos); // include ?
				strValue = prefix + strValue;
				Log.v(mConfig.getName() + ": ", "Link2 = " + strValue);
			}
			mSaleInfor.setLink(strValue);
			return true;
		}

		Log.v(mConfig.getName() + ": ", "can't not found Link");
		return false;
	}

	private boolean ParsePriceElement(Element elementParent)
	{
		if (mBookInfor.getPrice() > 0
				&& mBookInfor.getPriority() >= mConfig.getPriority())
		{
			Log.v("list price : ", "the existing is higher priority");
			return true;
		}		
		
		String strValue = ParseStringFromElement(elementParent, mConfig
				.getPriceElement());
		if (strValue == null)
		{
			Log.v(mConfig.getName() + ": ", "can't not found Price_Element");
			return false;
		}

		try
		{
			Log.v(mConfig.getName() + ": ", "Price_Element = " + strValue);
			String strValue2 = strValue.replaceAll("\\D", "");
			float fValue = Float.parseFloat(strValue2);
			mBookInfor.setPrice((float) (fValue / Math.pow(10, mConfig
					.getPriceElement().getDigits())));
			mBookInfor.setPriceUnit(mConfig.getUnit());
			mBookInfor.setPriority(mConfig.getPriority());
		}
		catch (Exception e)
		{
			return false;
		}

		return true;
	}

	protected boolean ParseDiscountPriceElement(Element elementParent)
	{
		String strValue = ParseStringFromElement(elementParent, mConfig
				.getDiscountElement());
		if (strValue == null)
		{
			Log.v(mConfig.getName() + ": ",
					"can't not found Discount_Price_Element");
			return false;
		}

		try
		{
			Log.v(mConfig.getName() + ": ", "Discount_Price_Element="
					+ strValue);
			String strValue2 = strValue.replaceAll("\\D", "");
			float fValue = Float.parseFloat(strValue2);
			mSaleInfor.setPrice((float) (fValue / Math.pow(10, mConfig
					.getDiscountElement().getDigits())));
		}
		catch (Exception e)
		{
			return false;
		}

		return true;
	}

	protected String ParseStringFromElement(Element elementParent,
			InformationElement config)
	{
		if (config.getPath() == null || config.getPath().length() == 0)
		{
			return "";
		}

		String value = "";
		Element element = (Element) XMLParseHelper.SelectSingleNode(config
				.getPath(), elementParent);
		if (element != null)
		{
			Log.v("get element Ok, path=", config.getPath());
			if (config.getValueType() == ShopConfiguration.ENodeType.eCONTENT)
			{
				value = XMLParseHelper.getTextContent(element);
			}
			else if (config.getValueType() == ShopConfiguration.ENodeType.eATTRIBUTE)
			{
				if (config.getAttribute() != null)
				{
					value = element.getAttribute(config.getAttribute());
				}
			}
			else if (config.getValueType() == ShopConfiguration.ENodeType.eSCRIPT)
			{
				if (config.getAttribute() != null)
				{
					String link = element.getAttribute(config.getAttribute());
					Log.v("exceute a http get command=", link);
					value = HttpConnectHelper.executeGetCommand(link);
				}
			}
		}
		else
		{
			Log.v("get element fails, path=", config.getPath());
		}
		
		// remove the Chinese/ANSI blank!
		try
		{
			while (value.length() > 0 && value.charAt(0) == '\u0020')
			{
				value = value.substring(1);
			}
			while (value.length() > 0
					&& value.charAt(value.length() - 1) == '\u3000')
			{
				value = value.substring(0, value.length() - 1);
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		return value.trim();
	}

	protected ShopConfiguration	mConfig;
	protected BookBasicInfor	mBookInfor;
	protected BookSaleInfor		mSaleInfor;
}