package luj.tools.BookBuyer;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import org.apache.http.NameValuePair;
import org.apache.http.message.BasicNameValuePair;

import luj.tools.BookBuyer.GlobalOption.EInquireMode;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.AlertDialog.Builder;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.DialogInterface.OnClickListener;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

public class BookBuyer extends Activity
{
	// buttons
	private Button				mAddByKeyboardBtn;
	private Button				mAddByCameraBtn;
	private Button				mInquireBtn;
	private Button				mOptionBtn;
	private Button				mAboutBtn;

	// list view for books
	private ListView			mlv;

	// table foot
	private TextView[]			mTitleOfShopTxt	= new TextView[4];
	private TextView			mStatusbarTxt;
	private TextView			mSumOfListPriceTxt;
	private TextView[]			mSumOfShopPriceTxt	= new TextView[4];
	
	// handler for updating the price statistics
	private Handler mUpdateUIHandler;

	// data adapter
	private BookTableAdapter	mAdapter;

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);

		GlobalOption.getInstance().setCurrentContext(getApplicationContext());
		
		// load global option
		DataExchange.getIntance().loadGlobalOption();
		ShopFactory.getInstance();

		// Button for respective functions
		mAddByKeyboardBtn = (Button) findViewById(R.id.MainDialog_Button_Add_By_Keyboard);
		mAddByKeyboardBtn.setOnClickListener(new AddByKeyboardButtonOnClickListener());
		
		mAddByCameraBtn = (Button) findViewById(R.id.MainDialog_Button_Add_By_Camera);
		mAddByCameraBtn.setOnClickListener(new AddByCameraButtonOnClickListener());

		mInquireBtn = (Button) findViewById(R.id.MainDialog_Button_Inquire);
		mInquireBtn.setOnClickListener(new InquireButtonOnClickListener());

		mOptionBtn = (Button) findViewById(R.id.MainDialog_Button_Option);
		mOptionBtn.setOnClickListener(new OptionButtonOnClickListener());

		mAboutBtn = (Button) findViewById(R.id.MainDialog_Button_About);
		mAboutBtn.setOnClickListener(new AboutButtonOnClickListener());

		// ListView for price
		mlv = (ListView) findViewById(R.id.Book_Table);
		LayoutInflater inflater = LayoutInflater.from(this);

		View header = inflater.inflate(R.layout.book_table_header, null);
		mTitleOfShopTxt[0] = (TextView)header.findViewById(R.id.Book_Table_Header_Shop0);
		mTitleOfShopTxt[1] = (TextView)header.findViewById(R.id.Book_Table_Header_Shop1);
		mTitleOfShopTxt[2] = (TextView)header.findViewById(R.id.Book_Table_Header_Shop2);
		mTitleOfShopTxt[3] = (TextView)header.findViewById(R.id.Book_Table_Header_Shop3);
		mlv.addHeaderView(header);

		View footer = inflater.inflate(R.layout.book_table_footer, null);
		mStatusbarTxt = (TextView)footer.findViewById(R.id.Book_Table_Footer_Statusbar);
		mSumOfListPriceTxt = (TextView)footer.findViewById(R.id.Book_Table_Footer_Sum_Listprice);
		mSumOfShopPriceTxt[0] = (TextView)footer.findViewById(R.id.Book_Table_Footer_Sum_Shop0);
		mSumOfShopPriceTxt[1] = (TextView)footer.findViewById(R.id.Book_Table_Footer_Sum_Shop1);
		mSumOfShopPriceTxt[2] = (TextView)footer.findViewById(R.id.Book_Table_Footer_Sum_Shop2);
		mSumOfShopPriceTxt[3] = (TextView)footer.findViewById(R.id.Book_Table_Footer_Sum_Shop3);
		mlv.addFooterView(footer);

		mUpdateUIHandler = new UpdatePriceStatistics();
		mAdapter = new BookTableAdapter(this, mUpdateUIHandler);
		mlv.setAdapter(mAdapter);

		// load initial data
		initializeData();
	}

	public void onStop()
	{
		DataExchange.getIntance().saveBookList();
		super.onStop();
	}
	
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent intent)
	{
		// super.onActivityResult(requestCode, resultCode, data);
		if (requestCode == 0)
		{
			if (resultCode == RESULT_OK)
			{
				String format = intent.getStringExtra("SCAN_RESULT_FORMAT");
				String contents = intent.getStringExtra("SCAN_RESULT");		
				Log.v("Format   : ", format);
				Log.v("Contents : ", contents);
				if (ISBNValidator.validate(contents))
				{
					if (Library.getInstance().addBook(contents))
					{
						mAdapter.notifyDataSetChanged();
					}
					else
					{
						Toast.makeText(BookBuyer.this, BookBuyer.this.getResources().getString(R.string.MSG_Existing_ISBN), Toast.LENGTH_SHORT).show();
					}
				}
				else
				{
					Toast.makeText(BookBuyer.this, BookBuyer.this.getResources().getString(R.string.MSG_Invalid_ISBN), Toast.LENGTH_SHORT).show();
				}
			}
		}
	}
	
	private boolean initializeData()
	{
//		Library.getInstance().addBook("9780307463630");
		Library.getInstance().addBook("9787805888767");
//		Library.getInstance().addBook("9787115200846");
//		Library.getInstance().addBook("9787115230218");

		DataExchange.getIntance().loadBookList();

		mUpdateUIHandler.sendMessage(new Message());

		return true;
	}

	private void updatePriceSum()
	{
		float fSumOfListPrice = 0.0f;
		float fSumOfDiscountPrice[] = new float[GlobalOption.MAX_NUMBER_SHOP];
		int nShopNum = GlobalOption.getInstance().getSelectedShops().size();
		if (nShopNum > GlobalOption.MAX_NUMBER_SHOP)
		{
			nShopNum = GlobalOption.MAX_NUMBER_SHOP;
		}
		
		// price
		int iBookCount = Library.getInstance().getBookCount();
		for (int i = 0; i < iBookCount; ++i)
		{
			Book book = Library.getInstance().getBook(i);
			if (book == null || !book.getSelected())
			{
				continue;
			}

			float fListPrice = book.getBasicInfor().getPrice();
			if (fListPrice > 0)
			{
				fSumOfListPrice += fListPrice;
			}
			else
			{
				continue;
			}

			for (int j = 0; j < nShopNum; ++j)
			{
				float fDiscountPrice = book.getSaleInfor(
						GlobalOption.getInstance().getSelectedShops().get(j))
						.getPrice();
				if (fDiscountPrice > 0)
				{
					fSumOfDiscountPrice[j] += fDiscountPrice;
				}
				else
				{
					fSumOfDiscountPrice[j] += fListPrice;
				}
			}
		}

		// title and sum
		mSumOfListPriceTxt.setText(String.format("%1.2f", fSumOfListPrice));
		for (int i = 0; i < GlobalOption.MAX_NUMBER_SHOP; ++i)
		{
			if (i < nShopNum)
			{
				mTitleOfShopTxt[i].setText(GlobalOption.getInstance().getSelectedShops().get(i));
				mSumOfShopPriceTxt[i].setText(String.format("%1.2f", fSumOfDiscountPrice[i]));
			}
			else
			{
				mTitleOfShopTxt[i].setText("");
				mSumOfShopPriceTxt[i].setText("");
			}
		}
	}
	
	private class UpdatePriceStatistics extends Handler
	{
        public UpdatePriceStatistics()
        {
        }

		public void handleMessage(Message msg)
		{
			super.handleMessage(msg);
			BookBuyer.this.updatePriceSum();
			BookBuyer.this.mAdapter.notifyDataSetChanged();
		}
	}

	private class ClientModeInquireThread extends
			AsyncTask<String, String, Integer>
	{
		protected Integer doInBackground(String... params)
		{
			int nShopNum = GlobalOption.getInstance().getSelectedShops().size();
			if (nShopNum > GlobalOption.MAX_NUMBER_SHOP)
			{
				nShopNum = GlobalOption.MAX_NUMBER_SHOP;
			}

			int iBookCount = Library.getInstance().getBookCount();
			for (int i = 0; i < iBookCount; ++i)
			{
				Book bookInfor = Library.getInstance().getBook(i);
				if (!bookInfor.getSelected())
				{
					continue;
				}
				for (int j = 0; j < nShopNum; ++j)
				{
					String shopName = GlobalOption.getInstance().getSelectedShops().get(j);
					
					// whether to update or remain old information
					boolean bUpdating = true;
					if (bookInfor.getSaleInfor(shopName).getPrice()>0)
					{
						long lCurrent = System.currentTimeMillis();
						long lOld = bookInfor.getSaleInfor(shopName).getTimeStamp();
						long lPassedHour = (lCurrent-lOld)/3600000;
						if (lPassedHour < GlobalOption.getInstance().getDataExpireHour())
						{
							bUpdating = false;
						}
					}
					
					// inquire
					if (bUpdating)
					{
						ShopInspector inspector = ShopFactory.getInstance().createShopInspector(shopName);
						if (inspector != null)
						{
							boolean bSuccess = inspector.Inquire(bookInfor.getBasicInfor(), bookInfor.getSaleInfor(shopName));
							if (bSuccess)
							{
								bookInfor.getSaleInfor(shopName).setTimeStamp(System.currentTimeMillis());
							}
						}
					}
					
					// update progress
					int progress = (int) ((i * nShopNum + j + 1.0f) / (iBookCount * nShopNum) * 100);
					String fmt = getResources().getString(R.string.MSG_Inquire_In_Progress);
					String msg = String.format(fmt, progress);
					publishProgress(msg);
					Log.v("Inquire thread", "done " + progress + "%");
				}
			}

			return 0;
		}
		
		protected void onPreExecute()
		{
			// update status bar
			String fmt = getResources().getString(R.string.MSG_Inquire_In_Progress);
			String msg = String.format(fmt, 0);
			mStatusbarTxt.setText(msg);

			// update price table
			mUpdateUIHandler.sendMessage(new Message());

			// disable Inquire button
			mInquireBtn.setEnabled(false);
		}		

		protected void onProgressUpdate(String... progress)
		{
			// percent_number
			mStatusbarTxt.setText(progress[0]);

			// update price table
			mUpdateUIHandler.sendMessage(new Message());
		}

		protected void onPostExecute(Integer result)
		{
			// update status bar
			mStatusbarTxt.setText(getResources().getString(
					R.string.MSG_Inquire_Finished));

			// update price table
			mUpdateUIHandler.sendMessage(new Message());

			// enable Inquire button
			mInquireBtn.setEnabled(true);
		}
	}

	private class CloudModeInquireThread extends
			AsyncTask<String, Integer, Integer>
	{
		protected Integer doInBackground(String... params)
		{
			int retValue = -1;

			List<NameValuePair> postParams = new ArrayList<NameValuePair>();
			postParams.add(new BasicNameValuePair("version", "1.0"));
			postParams.add(new BasicNameValuePair("type", "inquire"));
			String strContent = DataExchange.getIntance()
					.createClondInquireCommand();
			postParams.add(new BasicNameValuePair("content", strContent));
			String strXML = HttpConnectHelper.executePostCommand(GlobalOption
					.getInstance().getCloudLink(), postParams);
			if (strXML != null && strXML.length() != 0)
			{
				DataExchange.getIntance().loadBookFromString(strXML);
				retValue = 0;
			}

			return retValue;
		}
		
		protected void onPreExecute()
		{
			// update status bar
			String fmt = getResources().getString(R.string.MSG_Inquire_In_Progress);
			String msg = String.format(fmt, 0);
			mStatusbarTxt.setText(msg);

			// update price table
			mUpdateUIHandler.sendMessage(new Message());

			// disable Inquire button
			mInquireBtn.setEnabled(false);
		}

		protected void onPostExecute(Integer result)
		{
			// update status bar
			mStatusbarTxt.setText(getResources().getString(
					R.string.MSG_Inquire_Finished));

			// update price table
			mUpdateUIHandler.sendMessage(new Message());

			// enable Inquire button
			mInquireBtn.setEnabled(true);
		}
	}

	// OnClickListener for Add button
	private final class AddByKeyboardButtonOnClickListener implements
			Button.OnClickListener
	{
		public void onClick(View v)
		{
			LayoutInflater inflater = LayoutInflater.from(BookBuyer.this);
			View view = inflater.inflate(R.layout.dialog_input_isbn, null);
			final EditText edit = (EditText)view.findViewById(R.id.InputISBNDialog_Edit_Input_ISBN);
			AlertDialog.Builder builder = new AlertDialog.Builder(BookBuyer.this);
			builder.setTitle(BookBuyer.this.getResources().getString(R.string.InputISBNDialog_Title));  
			builder.setView(view);
			builder.setPositiveButton(BookBuyer.this.getResources().getString(R.string.InputISBNDialog_Button_Add), new OnClickListener()
			{				
				public void onClick(DialogInterface dialog, int which)
				{
					String isbn = edit.getText().toString();
					if (isbn.length() != 0)
					{
						Log.v("ISBN = ", isbn);
						if (ISBNValidator.validate(isbn))
						{
							if (Library.getInstance().addBook(isbn))
							{
								mAdapter.notifyDataSetChanged();
							}
							else
							{
								Toast.makeText(BookBuyer.this, BookBuyer.this.getResources().getString(R.string.MSG_Existing_ISBN), Toast.LENGTH_SHORT).show();
							}
						}
						else
						{
							Toast.makeText(BookBuyer.this, BookBuyer.this.getResources().getString(R.string.MSG_Invalid_ISBN), Toast.LENGTH_SHORT).show();
						}						
					}					
				}
			});
			builder.setNegativeButton(BookBuyer.this.getResources().getString(R.string.InputISBNDialog_Button_Go_Back), null);			
			builder.show();
		}
	}
	
	// OnClickListener for Add button
	private final class AddByCameraButtonOnClickListener implements
			Button.OnClickListener
	{
		public void onClick(View v)
		{
			Intent intent = new Intent("com.google.zxing.client.android.SCAN");
			intent.putExtra("SCAN_MODE", "PRODUCT_MODE");
			startActivityForResult(intent, 0);
		}
	}

	// OnClickListener for Check button
	private final class InquireButtonOnClickListener implements
			Button.OnClickListener
	{
		public void onClick(View v)
		{
			// there is nothing?
			if (Library.getInstance().getBookCount() == 0
					|| GlobalOption.getInstance().getSelectedShops().size() == 0)
			{
				return;
			}

			// new a thread to run Inquire process
			if (GlobalOption.getInstance().getInquireMode() == EInquireMode.eCloud)
			{
				new CloudModeInquireThread().execute("");
			}
			else
			{
				new ClientModeInquireThread().execute("");
			}
		}
	}

	// OnClickListener for Email button
	private final class OptionButtonOnClickListener implements
			Button.OnClickListener
	{
		public void onClick(View v)
		{
			Vector<String> shopsVector = ShopFactory.getInstance().getBuildinShopList();
			final String[] shops = (String[])shopsVector.toArray(new String[shopsVector.size()]);
            final boolean[] chooses = new boolean[shopsVector.size()];
            for (int i=0; i<shopsVector.size(); ++i)
            {
            	chooses[i] = GlobalOption.getInstance().getSelectedShops().contains(shops[i]);
            }
            
            Builder builder = new AlertDialog.Builder(BookBuyer.this);
            builder.setTitle(BookBuyer.this.getResources().getString(R.string.OptionDialog_Title));
            builder.setMultiChoiceItems(shops, chooses, new DialogInterface.OnMultiChoiceClickListener()
			{				
				public void onClick(DialogInterface dialog, int which, boolean isChecked)
				{
					chooses[which] = isChecked;
				}
			});
            builder.setPositiveButton(BookBuyer.this.getResources().getString(R.string.OptionDialog_Yes), new OnClickListener()
            {
    			public void onClick(DialogInterface dialog,	int which)
    			{
    				Vector<String> selectedShops = new Vector<String>();
    				for (int i=0; i<shops.length; ++i)
    				{
    					if (chooses[i])
    					{
    						selectedShops.add(shops[i]);
    					}
    				}
		            if (selectedShops.isEmpty())
		            {
		            	Toast.makeText(BookBuyer.this, BookBuyer.this.getResources().getString(R.string.MSG_None_Shop_Selected), Toast.LENGTH_SHORT).show();
		            }
		            else
		            {
		            	if (selectedShops.size()>GlobalOption.MAX_NUMBER_SHOP)
		            	{
		            		Toast.makeText(BookBuyer.this, BookBuyer.this.getResources().getString(R.string.MSG_Show_First4_Shops), Toast.LENGTH_SHORT).show();
		            	}
		            	GlobalOption.getInstance().setSelectedShops(selectedShops);
		            	DataExchange.getIntance().saveGlobalOption();
		            	mUpdateUIHandler.sendMessage(new Message());    		            	
		            }
    			}
            });
            builder.setNegativeButton(BookBuyer.this.getResources().getString(R.string.OptionDialog_No), null);
            builder.show();
		}
	}

	// OnClickListener for Help button
	private final class AboutButtonOnClickListener implements
			Button.OnClickListener
	{
		public void onClick(View v)
		{
			LayoutInflater inflater = LayoutInflater.from(BookBuyer.this);
			final View view = inflater.inflate(R.layout.dialog_about, null); 
			AlertDialog.Builder builder = new AlertDialog.Builder(BookBuyer.this);
			builder.setTitle(BookBuyer.this.getResources().getString(R.string.app_name));  
			builder.setView(view);  
			builder.setPositiveButton(BookBuyer.this.getResources().getString(R.string.AboutDialog_Button_OK), null);
			builder.show();
		}
	}
}