package luj.tools.BookBuyer;

import java.io.ByteArrayInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import luj.tools.BookBuyer.Book.BookBasicInfor;
import luj.tools.BookBuyer.Book.BookSaleInfor;
import luj.tools.BookBuyer.GlobalOption.EInquireMode;
import luj.tools.BookBuyer.ShopConfiguration.InformationElement;

import org.apache.http.NameValuePair;
import org.apache.http.message.BasicNameValuePair;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;
import org.xmlpull.v1.XmlSerializer;

import android.util.Xml;
import android.content.Context;

public class DataExchange
{
	public static DataExchange getIntance()
	{
		if (mInstance == null)
		{
			mInstance = new DataExchange();
		}
		return mInstance;
	}
	
	public void loadGlobalOption()
	{
		String xmlFile = GlobalOption.getInstance().getGlobalOptionFile();

		try
		{
			SAXParser parser = SAXParserFactory.newInstance().newSAXParser();
			Vector<String> shopList = new Vector<String>();
			GlobalOption.getInstance().setSelectedShops(shopList);
			OptionXMLHandler handler = new OptionXMLHandler(shopList);
			Context context = GlobalOption.getInstance().getCurrentContext();
			FileInputStream fis = context.openFileInput(xmlFile);
			parser.parse(fis, handler);			
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}
	
	private class OptionXMLHandler extends DefaultHandler
	{
		private Vector<String> mShopList;
		
		public OptionXMLHandler(Vector<String> shopList)
		{
			mShopList = shopList;
		}
		
		public void startElement(String uri, String localName, String name,
				Attributes attributes) throws SAXException
		{
			super.startElement(uri, localName, name, attributes);

			if (localName.equalsIgnoreCase("cloud"))
			{
				String link = attributes.getValue("link");
				if (link != null && link.length() != 0)
				{
					GlobalOption.getInstance().setInquireMode(EInquireMode.eCloud, link);
				}
			}
			else if (localName.equalsIgnoreCase("shop"))
			{		
				String shopName = attributes.getValue("name");
				if (shopName != null && shopName.length() != 0)
				{
					mShopList.add(shopName);
				}
			}
		}

		public void endElement(String uri, String localName, String name)
				throws SAXException
		{
		}
	}

	public void saveGlobalOption()
	{
		// output file name
		String xmlFile = GlobalOption.getInstance().getGlobalOptionFile();

		XmlSerializer serializer = Xml.newSerializer();
		StringWriter writer = new StringWriter();
		try
		{
			serializer.setOutput(writer);

			// <?xml>
			serializer.startDocument("UTF-8", true);

			// <library
			serializer.startTag("", "option");
			
			// cloud/client
			if (GlobalOption.getInstance().getInquireMode() == EInquireMode.eCloud)
			{
				serializer.startTag("", "cloud");
				String link = GlobalOption.getInstance().getCloudLink();
				serializer.attribute("", "link", link);
				serializer.startTag("", "cloud");
			}			

			// <shops>
			serializer.startTag("", "shops");
			int nShopNum = GlobalOption.getInstance().getSelectedShops().size();
			for (int i = 0; i < nShopNum; ++i)
			{
				// shop
				String shopName = GlobalOption.getInstance().getSelectedShops().get(i);
				serializer.startTag("", "shop");
				serializer.attribute("", "name", shopName);
				serializer.endTag("", "shop");
			}
			serializer.endTag("", "shops");
			
			// end option
			serializer.endTag("", "option");

			// end Document
			serializer.endDocument();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		try
		{
			Context context = GlobalOption.getInstance().getCurrentContext();
			FileOutputStream fos = context.openFileOutput(xmlFile,
					Context.MODE_PRIVATE);
			fos.write(writer.toString().getBytes());
			fos.close();
		}
		catch (FileNotFoundException e)
		{
			e.printStackTrace();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
	}
	
	public void loadBookList()
	{
		String xmlFile = GlobalOption.getInstance().getLibraryFile();

		try
		{
			SAXParser parser = SAXParserFactory.newInstance().newSAXParser();
			LibraryXMLHandler handler = new LibraryXMLHandler();
			Context context = GlobalOption.getInstance().getCurrentContext();
			FileInputStream fis = context.openFileInput(xmlFile);
			parser.parse(fis, handler);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	private class LibraryXMLHandler extends DefaultHandler
	{
		private Book	mBook	= null;

		public void startElement(String uri, String localName, String name,
				Attributes attributes) throws SAXException
		{
			super.startElement(uri, localName, name, attributes);

			if (localName.equalsIgnoreCase("book"))
			{
				String isbn = attributes.getValue("isbn");
				if (isbn != null && isbn.length() != 0)
				{
					Library.getInstance().addBook(isbn);
					mBook = Library.getInstance().getBook(isbn);
					if (mBook == null)
					{
						return;
					}

					String strValue = attributes.getValue("selected");
					if (strValue != null)
					{
						if (strValue.equalsIgnoreCase("true"))
						{
							mBook.setSelected(true);
						}
						else
						{
							mBook.setSelected(false);
						}
					}

					BookBasicInfor baseInfor = mBook.getBasicInfor();

					baseInfor.setName(attributes.getValue("name"));
					baseInfor.setAuthor(attributes.getValue("author"));
					baseInfor.setPublisher(attributes.getValue("publisher"));

					if (attributes.getValue("price") != null)
					{
						try
						{
							float fValue = Float.parseFloat(attributes
									.getValue("price"));
							baseInfor.setPrice(fValue);
						}
						catch (Exception e)
						{
							e.printStackTrace();
						}
					}
				}
			}
			else if (localName.equalsIgnoreCase("shop"))
			{
				if (mBook == null)
				{
					return;
				}

				String shopName = attributes.getValue("name");
				if (shopName != null && shopName.length() != 0)
				{
					BookSaleInfor saleInfor = mBook.getSaleInfor(shopName);
					saleInfor.setLink(attributes.getValue("link"));
					if (attributes.getValue("discount") != null)
					{
						try
						{
							float fValue = Float.parseFloat(attributes
									.getValue("discount"));
							saleInfor.setPrice(fValue);
						}
						catch (Exception e)
						{
							e.printStackTrace();
						}
					}
				}
			}
		}

		public void endElement(String uri, String localName, String name)
				throws SAXException
		{
			if (localName.equalsIgnoreCase("book"))
			{
				mBook = null;
			}
		}
	}

	public void saveBookList()
	{
		XmlSerializer serializer = Xml.newSerializer();
		StringWriter writer = new StringWriter();
		try
		{
			serializer.setOutput(writer);

			// <?xml>
			serializer.startDocument("UTF-8", true);

			// <library
			serializer.startTag("", "library");

			// <book>
			int iBookCount = Library.getInstance().getBookCount();
			for (int i = 0; i < iBookCount; ++i)
			{
				Book book = Library.getInstance().getBook(i);
				if (book == null)
				{
					continue;
				}

				serializer.startTag("", "book");

				BookBasicInfor baseInfor = book.getBasicInfor();
				serializer.attribute("", "selected",
						book.getSelected() ? "true" : "false");
				serializer.attribute("", "isbn", baseInfor.getISBN());

				if (baseInfor.getName() != null)
				{
					serializer.attribute("", "name", baseInfor.getName());
				}

				if (baseInfor.getAuthor() != null)
				{
					serializer.attribute("", "author", baseInfor.getAuthor());
				}

				if (baseInfor.getPublisher() != null)
				{
					serializer.attribute("", "publisher", baseInfor
							.getPublisher());
				}

				serializer.attribute("", "price", String.format("%1.2f",
						baseInfor.getPrice()));

				for (Map.Entry<String, BookSaleInfor> m : book.getSaleInfor()
						.entrySet())
				{
					serializer.startTag("", "shop");

					serializer.attribute("", "name", m.getKey());
					serializer.attribute("", "discount", String.format("%1.2f",
							m.getValue().getPrice()));
					if (m.getValue().getLink() != null)
					{
						serializer
								.attribute("", "link", m.getValue().getLink());
					}

					serializer.endTag("", "shop");
				}

				serializer.endTag("", "book");
			}

			// </library>
			serializer.endTag("", "library");

			// end Document
			serializer.endDocument();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		try
		{
			Context context = GlobalOption.getInstance().getCurrentContext();
			FileOutputStream fos = context.openFileOutput(GlobalOption.getInstance().getLibraryFile(),
					Context.MODE_PRIVATE);
			fos.write(writer.toString().getBytes());
			fos.close();
		}
		catch (FileNotFoundException e)
		{
			e.printStackTrace();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
	}

	public String createClondInquireCommand()
	{
		XmlSerializer serializer = Xml.newSerializer();
		StringWriter writer = new StringWriter();
		try
		{
			serializer.setOutput(writer);

			// <?xml>
			serializer.startDocument("UTF-8", true);

			// <library
			serializer.startTag("", "CloneInquireCommand");

			// books
			int iBookCount = Library.getInstance().getBookCount();
			for (int i = 0; i < iBookCount; ++i)
			{
				Book bookInfor = Library.getInstance().getBook(i);
				if (bookInfor.getSelected())
				{
					serializer.startTag("", "book");
					serializer.attribute("", "isbn", bookInfor.getBasicInfor()
							.getISBN());
					serializer.endTag("", "book");
				}
			}

			// shop list
			int nShopNum = GlobalOption.getInstance().getSelectedShops().size();
			if (nShopNum > GlobalOption.MAX_NUMBER_SHOP)
			{
				nShopNum = GlobalOption.MAX_NUMBER_SHOP;
			}
			for (int i = 0; i < nShopNum; ++i)
			{
				serializer.startTag("", "shop");
				serializer.attribute("", "name", GlobalOption.getInstance()
						.getSelectedShops().get(i));
				serializer.endTag("", "shop");
			}

			// </library>
			serializer.endTag("", "CloneInquireCommand");

			// end Document
			serializer.endDocument();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		return writer.toString();
	}

	public void loadBookFromString(String strXML)
	{
		try
		{
			SAXParser parser = SAXParserFactory.newInstance().newSAXParser();
			LibraryXMLHandler handler = new LibraryXMLHandler();
			InputStream fis = new ByteArrayInputStream(strXML.getBytes());
			parser.parse(fis, handler);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}	
	
	private class ShopConfigurationXMLHandler extends DefaultHandler
	{
		private Map<String, ShopConfiguration>	mShopConfigs;
		private Vector<String>					mShopList;
		private ShopConfiguration				mShopCfg		= 	null;
		
		public ShopConfigurationXMLHandler(Map<String, ShopConfiguration> shopConfigs, Vector<String> shopList)
		{
			mShopConfigs = shopConfigs;
			mShopList = shopList;
		}

		public void startElement(String uri, String localName, String name,
				Attributes attributes) throws SAXException
		{
			super.startElement(uri, localName, name, attributes);

			// shop node
			if (localName.equalsIgnoreCase("shop"))
			{	
				// reset shop to null
				mShopCfg = null;
				
				// if the shop is not existing, create it.
				String shopName = attributes.getValue("id");
				if (shopName != null && shopName.length() != 0)
				{
					// ignore the example shop
					if (shopName.contains("_test"))
					{
						return;
					}
					
					if (!mShopConfigs.containsKey(shopName))
					{
						mShopCfg = new ShopConfiguration(shopName);
					}
					else
					{
						return; //already existing!
					}
				}
				else
				{
					return; // invalid parameter
				}
				
				// is generic
				String isGeneric = attributes.getValue("generic");
				if (isGeneric != null && isGeneric.equalsIgnoreCase("no"))
				{
					mShopCfg.setGeneric(false);
				}				
				
				// read description
				String shopDescription = attributes.getValue("description");
				if (shopDescription != null && shopDescription.length() != 0)
				{
					mShopCfg.setDescription(shopDescription);
				}
				
				// priority, show order
				String shopPriority = attributes.getValue("priority");
				if (shopPriority != null && shopPriority.length() != 0)
				{
					try
					{
						mShopCfg.setPriority(Integer.parseInt(shopPriority));
					}
					catch (Exception e)
					{
						e.printStackTrace();
					}
				}
				
				// read unit
				String unit = attributes.getValue("unit");
				if (unit != null && unit.length() != 0)
				{
					mShopCfg.setUnit(unit);
				}
				
				return;
			}
			
			if (mShopCfg == null)
			{
				return;
			}
			
			// http connection information
			if (localName.equalsIgnoreCase("http"))
			{
				String protocol = attributes.getValue("protocol");
				if (protocol == null)
				{
					mShopCfg = null;
					return;
				}
				
				if (protocol.equalsIgnoreCase("get"))
				{
					String link = attributes.getValue("link");
					if (link != null && link.length() != 0)
					{
						mShopCfg.setHttpGetLink(link);
						return;
					}
					else
					{
						mShopCfg = null;
						return;
					}
				}
				else if (protocol.equalsIgnoreCase("post"))
				{
					String link = attributes.getValue("link");
					if (link != null && link.length() != 0)
					{
						mShopCfg.setHttpPostLink(link);
						List<NameValuePair> params = new ArrayList<NameValuePair>();
						for (int i=0; i<attributes.getLength(); ++i)
						{
							String key = attributes.getLocalName(i);
							String value = attributes.getValue(i);
							if (!key.equalsIgnoreCase("protocol") && !key.equalsIgnoreCase("link"))
							{
								params.add(new BasicNameValuePair(key, value));
							}
						}
						mShopCfg.setHttpPostParams(params);
						return;
					}
					else
					{
						mShopCfg = null;
						return;
					}
				}
				else
				{
					mShopCfg = null;
					return;
				}
			}
			// jump, the real page need get twice
			else if (localName.equalsIgnoreCase("jump"))
			{
				fillInforElement(attributes, mShopCfg.getJumpElement());
			}			
			// book entry
			else if (localName.equalsIgnoreCase("book"))
			{
				// should complete correct!!
				if (!fillInforElement(attributes, mShopCfg.getBookElement()))
				{
					mShopCfg = null;
					return;
				}
			}
			// ISBN
			else if (localName.equalsIgnoreCase("isbn"))
			{
				// optional, it may be empty!
				fillInforElement(attributes, mShopCfg.getISBNElement());
			}
			// book name
			else if (localName.equalsIgnoreCase("name"))
			{
				// optional, it may be empty!
				fillInforElement(attributes, mShopCfg.getBooknamElement());
			}
			// author
			else if (localName.equalsIgnoreCase("author"))
			{
				// optional, it may be empty!
				fillInforElement(attributes, mShopCfg.getAuthorlement());
			}
			// publisher
			else if (localName.equalsIgnoreCase("publisher"))
			{
				// optional, it may be empty!
				fillInforElement(attributes, mShopCfg.getPublisherElement());			
			}
			// link
			else if (localName.equalsIgnoreCase("link"))
			{
				// optional, it may be empty!
				fillInforElement(attributes, mShopCfg.getLinkElement());					
			}
			// price
			else if (localName.equalsIgnoreCase("price"))
			{
				// optional, it may be empty!
				fillInforElement(attributes, mShopCfg.getPriceElement());					
			}
			// discount
			else if (localName.equalsIgnoreCase("discount"))
			{
				// optional, it may be empty!
				fillInforElement(attributes, mShopCfg.getDiscountElement());					
			}			
		}

		public void endElement(String uri, String localName, String name)
				throws SAXException
		{
			if (localName.equalsIgnoreCase("shop"))
			{
				if (mShopCfg != null)
				{
					mShopConfigs.put(mShopCfg.getName(), mShopCfg);
					int position = 0;
					for (int i=0; i<mShopList.size(); ++i)
					{
						if (mShopCfg.getPriority() > mShopConfigs.get(mShopList.get(i)).getPriority())
						{
							break;
						}
						else
						{
							++position;
						}
					}
					mShopList.add(position, mShopCfg.getName());
					mShopCfg = null;
				}
			}
		}
		
		private boolean fillInforElement(Attributes attributes, InformationElement element)
		{
			String path = attributes.getValue("path");
			if (path != null && path.length() != 0)
			{
				// path
				element.setPath(path);
				
				// value type
				String value_type = attributes.getValue("value_type");
				if (value_type != null && value_type.length() != 0)
				{
					if (value_type.equalsIgnoreCase(ShopConfiguration.CONTENT))
					{
						element.setValueType(ShopConfiguration.ENodeType.eCONTENT);
					}
					else if (value_type.equalsIgnoreCase(ShopConfiguration.ATTRIBUTE))
					{
						element.setValueType(ShopConfiguration.ENodeType.eATTRIBUTE);
					}
					else if (value_type.equalsIgnoreCase(ShopConfiguration.SCRIPT))
					{
						element.setValueType(ShopConfiguration.ENodeType.eSCRIPT);
					}
					else
					{
						return false;
					}
				}
				
				// if it's Attribute, get attribute name
				String attribute_name = attributes.getValue("attribute_name");
				if (value_type != null && value_type.length() != 0)
				{
					element.setAttribute(attribute_name);
				}
				
				// try to find the decimal number
				String decimal = attributes.getValue("decimal");
				if (decimal != null && decimal.length() != 0)
				{
					try
					{
						int iNum = Integer.parseInt(decimal);
						element.setDigits(iNum);
					}
					catch (Exception e)
					{
						e.printStackTrace();
					}
				}
				
				return true;
			}
			else
			{
				return false;
			}
		}
	}

	public void loadShopConfiguration(Map<String, ShopConfiguration> shopConfigs, Vector<String> shopList)
	{
		try
		{
			SAXParser parser = SAXParserFactory.newInstance().newSAXParser();
			ShopConfigurationXMLHandler handler = new ShopConfigurationXMLHandler(shopConfigs, shopList);
			Context context = GlobalOption.getInstance().getCurrentContext();
			
			// load the user defined shop_configuration.xml
			try
			{
				FileInputStream fis = context.openFileInput(GlobalOption.getInstance().getShopConfigFile());
				parser.parse(fis, handler);
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
			
			// load build-in shop_configuration.xml
			InputStream  fis = context.getResources().openRawResource(R.raw.shop_config);
			parser.parse(fis, handler);	
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	private static DataExchange	mInstance;
}